@@ -16,7 +16,12 @@
 # include <memcheck.h>
 #endif
 
+//#define SURFDBG 1
+
 static Evas_Cache_Image * eci = NULL;
+#ifdef EVAS_CSERVE2
+static Evas_Cache2      * eci2 = NULL;
+#endif
 static int                reference = 0;
 
 /* static RGBA_Image *evas_rgba_line_buffer = NULL; */
@@ -76,11 +81,38 @@ static const Evas_Cache_Image_Func      _evas_common_image_func =
   NULL
 };
 
+#ifdef EVAS_CSERVE2
+static const Evas_Cache2_Image_Func      _evas_common_image_func2 =
+{
+  // _evas_common_rgba_image_new,
+  // _evas_common_rgba_image_delete,
+  _evas_common_rgba_image_surface_alloc,
+  _evas_common_rgba_image_surface_delete,
+  _evas_common_rgba_image_surface_pixels,
+  // evas_common_load_rgba_image_module_from_file,
+  // _evas_common_rgba_image_unload,
+  NULL, // _evas_common_rgba_image_dirty_region,
+  NULL, // _evas_common_rgba_image_dirty,
+  evas_common_rgba_image_size_set,
+  evas_common_rgba_image_from_copied_data,
+  evas_common_rgba_image_from_data,
+  NULL, // evas_common_rgba_image_colorspace_set,
+  // evas_common_load_rgba_image_data_from_file,
+  _evas_common_rgba_image_ram_usage,
+/*   _evas_common_rgba_image_debug */
+  NULL
+};
+#endif
+
 EAPI void
 evas_common_image_init(void)
 {
    if (!eci)
      eci = evas_cache_image_init(&_evas_common_image_func);
+#ifdef EVAS_CSERVE2
+   if (!eci2)
+     eci2 = evas_cache2_init(&_evas_common_image_func2);
+#endif
    reference++;
 ////   ERR("REF++=%i", reference);
 
@@ -113,6 +145,10 @@ evas_common_image_shutdown(void)
 // ENABLE IT AGAIN, hope it is fixed. Gustavo @ January 22nd, 2009.
        evas_cache_image_shutdown(eci);
        eci = NULL;
+#ifdef EVAS_CSERVE2
+       evas_cache2_shutdown(eci2);
+       eci2 = NULL;
+#endif
      }
 
 #ifdef BUILD_LOADER_EET
@@ -137,29 +173,20 @@ _evas_common_rgba_image_new(void)
    if (!im) return NULL;
    im->flags = RGBA_IMAGE_NOTHING;
    im->ref = 1;
-#ifdef EVAS_FRAME_QUEUING
-   LKI(im->cache_entry.ref_fq_add);
-   LKI(im->cache_entry.ref_fq_del);
-   pthread_cond_init(&(im->cache_entry.cond_fq_del), NULL);
-#endif
 
    evas_common_rgba_image_scalecache_init(&im->cache_entry);
+
    return &im->cache_entry;
 }
 
 static void
 _evas_common_rgba_image_delete(Image_Entry *ie)
 {
-   RGBA_Image   *im = (RGBA_Image *) ie;
+   RGBA_Image *im = (RGBA_Image *)ie;
 
 #ifdef BUILD_PIPE_RENDER
    evas_common_pipe_free(im);
-# ifdef EVAS_FRAME_QUEUING
-   LKD(im->cache_entry.ref_fq_add);
-   LKD(im->cache_entry.ref_fq_del);
-   pthread_cond_destroy(&(im->cache_entry.cond_fq_del));
-# endif
-#endif   
+#endif
    evas_common_rgba_image_scalecache_shutdown(&im->cache_entry);
    if (ie->info.module) evas_module_unref((Evas_Module *)ie->info.module);
    /* memset the image to 0x99 because i recently saw a segv where an
@@ -171,7 +198,41 @@ _evas_common_rgba_image_delete(Image_Entry *ie)
 //   memset(im, 0x99, sizeof(im));
 #ifdef EVAS_CSERVE
    if (ie->data1) evas_cserve_image_free(ie);
-#endif   
+#endif
+#ifdef EVAS_CSERVE2
+   if (ie->data1)
+       ERR("Shouldn't reach this point since we are using cache2: '%s' '%s'",
+           ie->file, ie->key);
+   // if (ie->data1) evas_cserve2_image_free(ie);
+#endif
+/*
+ * FIXME: This doesn't seem to be needed... But I'm not sure why.
+ *	 -- nash
+     {
+        Filtered_Image *fi;
+
+        EINA_LIST_FREE(im->filtered, fi)
+          {
+             free(fi->key);
+             _evas_common_rgba_image_delete((Image_Entry *)(fi->image));
+             free(fi);
+          }
+     }
+*/
+   if (ie->frames)
+     {
+        Eina_List *l;
+        Image_Entry_Frame *frame;
+        EINA_LIST_FOREACH(ie->frames, l, frame)
+          {
+           if (frame)
+             {
+                if (frame->data) free(frame->data);
+                if (frame->info) free(frame->info);
+                free (frame);
+             }
+          }
+     }
    free(im);
 }
 
@@ -182,6 +243,30 @@ evas_common_rgba_image_free(Image_Entry *ie)
    _evas_common_rgba_image_delete(ie);
 }
 
+#ifdef SURFDBG
+static Eina_List *surfs = NULL;
+
+static void
+surf_debug(void)
+{
+   Eina_List *l;
+   Image_Entry *ie;
+   RGBA_Image *im;
+   int i = 0;
+   
+   printf("----SURFS----\n");
+   EINA_LIST_FOREACH(surfs, l, ie)
+     {
+        im = ie;
+        printf("%i - %p - %ix%i  [%s][%s]\n", 
+               i, im->image.data, ie->allocated.w, ie->allocated.h,
+               ie->file, ie->key
+              );
+        i++;
+     }
+}
+#endif
+
 EAPI void
 evas_common_rgba_image_unload(Image_Entry *ie)
 {
@@ -219,10 +304,31 @@ evas_common_rgba_image_unload(Image_Entry *ie)
 #endif
         return;
      }
-#endif   
-   
+#endif
+
+#ifdef EVAS_CSERVE2
+   if (ie->data1)
+     {
+        ERR("Shouldn't reach this point since we are using cache2.");
+//         evas_cserve2_image_unload(ie);
+//         im->image.data = NULL;
+//         ie->allocated.w = 0;
+//         ie->allocated.h = 0;
+//         ie->flags.loaded = 0;
+#ifdef BUILD_ASYNC_PRELOAD
+        ie->flags.preload_done = 0;
+#endif
+        return;
+     }
+#endif
+
    if (im->image.data && !im->image.no_free)
-     free(im->image.data);
+     {
+        free(im->image.data);
+#ifdef SURFDBG
+        surfs = eina_list_remove(surfs, ie);
+#endif        
+     }
    im->image.data = NULL;
    ie->allocated.w = 0;
    ie->allocated.h = 0;
@@ -230,6 +336,49 @@ evas_common_rgba_image_unload(Image_Entry *ie)
 #ifdef BUILD_ASYNC_PRELOAD
    ie->flags.preload_done = 0;
 #endif
+#ifdef SURFDBG
+   surf_debug();
+#endif   
+}
+
+void
+_evas_common_rgba_image_post_surface(Image_Entry *ie)
+{
+#ifdef HAVE_PIXMAN
+# ifdef PIXMAN_IMAGE   
+   RGBA_Image *im = (RGBA_Image *)ie;
+
+   if (im->pixman.im) pixman_image_unref(im->pixman.im);
+   if (im->cache_entry.flags.alpha)
+     {
+        im->pixman.im = pixman_image_create_bits
+        (
+// FIXME: endianess determines this
+            PIXMAN_a8r8g8b8,
+//            PIXMAN_b8g8r8a8,
+            im->cache_entry.w, im->cache_entry.h,
+            im->image.data,
+            im->cache_entry.w * 4
+        );
+     }
+   else
+     {
+        im->pixman.im = pixman_image_create_bits
+        (
+// FIXME: endianess determines this
+            PIXMAN_x8r8g8b8,
+//            PIXMAN_b8g8r8x8,
+            im->cache_entry.w, im->cache_entry.h,
+            im->image.data,
+            im->cache_entry.w * 4
+        );
+     }
+# else
+   (void)ie;
+# endif
+#else
+   (void)ie;
+#endif
 }
 
 static int
@@ -240,7 +389,10 @@ _evas_common_rgba_image_surface_alloc(Image_Entry *ie, unsigned int w, unsigned
 
 #ifdef EVAS_CSERVE
    if (ie->data1) return 0;
-#endif   
+#endif
+#ifdef EVAS_CSERVE2
+   if (ie->data1) return 0;
+#endif
    if (im->image.no_free) return 0;
 
    if (im->flags & RGBA_IMAGE_ALPHA_ONLY)
@@ -248,10 +400,20 @@ _evas_common_rgba_image_surface_alloc(Image_Entry *ie, unsigned int w, unsigned
    else
      siz = w * h * sizeof(DATA32);
 
-   if (im->image.data) free(im->image.data);
+   if (im->image.data)
+     {
+        free(im->image.data);
+#ifdef SURFDBG
+        surfs = eina_list_remove(surfs, ie);
+#endif        
+     }
    im->image.data = malloc(siz);
    if (!im->image.data) return -1;
-
+   ie->allocated.w = w;
+   ie->allocated.h = h;
+#ifdef SURFDBG
+   surfs = eina_list_append(surfs, ie);
+#endif   
 #ifdef HAVE_VALGRIND
 # ifdef VALGRIND_MAKE_READABLE
    VALGRIND_MAKE_READABLE(im->image.data, siz);
@@ -261,7 +423,10 @@ _evas_common_rgba_image_surface_alloc(Image_Entry *ie, unsigned int w, unsigned
 #  endif
 # endif
 #endif
-
+   _evas_common_rgba_image_post_surface(ie);
+#ifdef SURFDBG
+   surf_debug();
+#endif
    return 0;
 }
 
@@ -270,6 +435,15 @@ _evas_common_rgba_image_surface_delete(Image_Entry *ie)
 {
    RGBA_Image   *im = (RGBA_Image *) ie;
 
+#ifdef HAVE_PIXMAN
+# ifdef PIXMAN_IMAGE   
+   if (im->pixman.im)
+     {
+        pixman_image_unref(im->pixman.im);
+        im->pixman.im = NULL;
+     }
+# endif   
+#endif
    if (ie->file)
      DBG("unload: [%p] %s %s", ie, ie->file, ie->key);
    if ((im->cs.data) && (im->image.data))
@@ -286,11 +460,22 @@ _evas_common_rgba_image_surface_delete(Image_Entry *ie)
    im->cs.data = NULL;
 
    if (im->image.data && !im->image.no_free)
-     free(im->image.data);
+     {
+        free(im->image.data);
+#ifdef SURFDBG
+        surfs = eina_list_remove(surfs, ie);
+#endif        
+     }
 #ifdef EVAS_CSERVE
    else if (ie->data1)
      evas_cserve_image_free(ie);
-#endif   
+#endif
+// #ifdef EVAS_CSERVE2
+//    else if (ie->data1)
+//      ERR("Shouldn't reach this point since we are using cache2.");
+//      // evas_cserve2_image_free(ie);
+// #endif
+
    im->image.data = NULL;
    ie->allocated.w = 0;
    ie->allocated.h = 0;
@@ -299,6 +484,9 @@ _evas_common_rgba_image_surface_delete(Image_Entry *ie)
 #endif
    ie->flags.loaded = 0;
    evas_common_rgba_image_scalecache_dirty(&im->cache_entry);
+#ifdef SURFDBG
+   surf_debug();
+#endif   
 }
 
 static void
@@ -316,7 +504,11 @@ _evas_common_rgba_image_dirty_region(Image_Entry* ie, unsigned int x __UNUSED__,
 
 #ifdef EVAS_CSERVE
    if (ie->data1) evas_cserve_image_free(ie);
-#endif   
+#endif
+#ifdef EVAS_CSERVE2
+   // if (ie->data1) evas_cserve2_image_free(ie);
+   if (ie->data1) ERR("Shouldn't reach this point since we are using cache2.");
+#endif
    im->flags |= RGBA_IMAGE_IS_DIRTY;
    evas_common_rgba_image_scalecache_dirty(&im->cache_entry);
 }
@@ -336,13 +528,21 @@ _evas_common_rgba_image_dirty(Image_Entry *ie_dst, const Image_Entry *ie_src)
      {
 #ifdef EVAS_CSERVE
         if (ie_src->data1) evas_cserve_image_free((Image_Entry*) ie_src);
-#endif        
+#endif
+#ifdef EVAS_CSERVE2
+        // if (ie_src->data1) evas_cserve2_image_free((Image_Entry*) ie_src);
+        if (ie_src->data1) ERR("Shouldn't reach this point since we are using cache2.");
+#endif
         return 1;
      }
 
 #ifdef EVAS_CSERVE
    if (ie_src->data1) evas_cserve_image_free((Image_Entry*) ie_src);
-#endif   
+#endif
+#ifdef EVAS_CSERVE2
+        // if (ie_src->data1) evas_cserve2_image_free((Image_Entry*) ie_src);
+        if (ie_src->data1) ERR("Shouldn't reach this point since we are using cache2.");
+#endif
    evas_common_image_colorspace_normalize(src);
    evas_common_image_colorspace_normalize(dst);
 /*    evas_common_blit_rectangle(src, dst, 0, 0, src->cache_entry.w, src->cache_entry.h, 0, 0); */
@@ -354,19 +554,22 @@ _evas_common_rgba_image_dirty(Image_Entry *ie_dst, const Image_Entry *ie_src)
 static int
 _evas_common_rgba_image_ram_usage(Image_Entry *ie)
 {
-   RGBA_Image   *im = (RGBA_Image *) ie;
-   int size = 0;
+   RGBA_Image *im = (RGBA_Image *)ie;
+   int size = sizeof(struct _RGBA_Image);
+
+   if (ie->cache_key) size += strlen(ie->cache_key);
+   if (ie->file) size += strlen(ie->file);
+   if (ie->key) size += strlen(ie->key);
 
-//   ram += sizeof(struct _RGBA_Image);
-//   if (im->info.real_file) ram += strlen(im->info.real_file);
-//   if (im->info.comment) ram += strlen(im->info.comment);
    if (im->image.data)
      {
-#ifdef EVAS_CSERVE
+#if defined(EVAS_CSERVE)
+        if ((!im->image.no_free) || (ie->data1))
+#elif defined(EVAS_CSERVE2)
         if ((!im->image.no_free) || (ie->data1))
 #else
         if ((!im->image.no_free))
-#endif          
+#endif
           size += im->cache_entry.w * im->cache_entry.h * sizeof(DATA32);
      }
    size += evas_common_rgba_image_scalecache_usage_get(&im->cache_entry);
@@ -514,8 +717,20 @@ evas_common_image_colorspace_normalize(RGBA_Image *im)
 	  {
 #ifdef EVAS_CSERVE
              if (((Image_Entry *)im)->data1) evas_cserve_image_free(&im->cache_entry);
-#endif             
-	     if (!im->image.no_free) free(im->image.data);
+#endif
+#ifdef EVAS_CSERVE2
+             // if (((Image_Entry *)im)->data1) evas_cserve2_image_free(&im->cache_entry);
+             if (((Image_Entry *)im)->data1) ERR("Shouldn't reach this point since we are using cache2.");
+#endif
+	     if (!im->image.no_free)
+               {
+                  free(im->image.data);
+#ifdef SURFDBG
+                  surfs = eina_list_remove(surfs, im);
+#endif                  
+                  ((Image_Entry *)im)->allocated.w = 0;
+                  ((Image_Entry *)im)->allocated.h = 0;
+               }
 	     im->image.data = im->cs.data;
 	     im->cs.no_free = im->image.no_free;
 	  }
@@ -527,10 +742,34 @@ evas_common_image_colorspace_normalize(RGBA_Image *im)
 						im->cache_entry.w, im->cache_entry.h);
 #endif
 	break;
+      case EVAS_COLORSPACE_YCBCR422601_PL:
+#ifdef BUILD_CONVERT_YUV
+        if ((im->image.data) && (*((unsigned char **)im->cs.data)))
+          evas_common_convert_yuv_422_601_rgba(im->cs.data, (DATA8*) im->image.data,
+                                               im->cache_entry.w, im->cache_entry.h);
+#endif
+        break;
+      case EVAS_COLORSPACE_YCBCR420NV12601_PL:
+#ifdef BUILD_CONVERT_YUV
+        if ((im->image.data) && (*((unsigned char **)im->cs.data)))
+          evas_common_convert_yuv_420_601_rgba(im->cs.data, (DATA8*) im->image.data,
+                                               im->cache_entry.w, im->cache_entry.h);
+#endif
+        break;
+      case EVAS_COLORSPACE_YCBCR420TM12601_PL:
+#ifdef BUILD_CONVERT_YUV
+        if ((im->image.data) && (*((unsigned char **)im->cs.data)))
+          evas_common_convert_yuv_420T_601_rgba(im->cs.data, (DATA8*) im->image.data,
+                                                im->cache_entry.w, im->cache_entry.h);
+#endif
+         break;
       default:
 	break;
      }
    im->cs.dirty = 0;
+#ifdef SURFDBG
+   surf_debug();
+#endif   
 }
 
 EAPI void
@@ -538,6 +777,16 @@ evas_common_image_colorspace_dirty(RGBA_Image *im)
 {
    im->cs.dirty = 1;
    evas_common_rgba_image_scalecache_dirty(&im->cache_entry);
+#ifdef HAVE_PIXMAN
+# ifdef PIXMAN_IMAGE   
+   if (im->pixman.im)
+     {
+        pixman_image_unref(im->pixman.im);
+        im->pixman.im = NULL;
+     }
+   _evas_common_rgba_image_post_surface((Image_Entry *)im);
+# endif   
+#endif
 }
 
 EAPI void
@@ -545,6 +794,10 @@ evas_common_image_set_cache(unsigned int size)
 {
    if (eci)
      evas_cache_image_set(eci, size);
+#ifdef EVAS_CSERVE2
+   if (eci2)
+     evas_cache2_limit_set(eci2, size);
+#endif
 }
 
 EAPI int
@@ -576,6 +829,14 @@ evas_common_image_cache_get(void)
    return eci;
 }
 
+#ifdef EVAS_CSERVE2
+EAPI Evas_Cache2*
+evas_common_image_cache2_get(void)
+{
+   return eci2;
+}
+#endif
+
 EAPI RGBA_Image *
 evas_common_image_line_buffer_obtain(int len)
 {
